Sketchup::require 'sketchup'

=begin

Message: 4
Date: Fri, 21 Sep 2012 17:57:17 +0200
From: Martin Vymazal <martin.vymazal@vki.ac.be>
To: gmsh@geuz.org
Subject: Re: [Gmsh] SketchUp to GEO file: line loop question
Message-ID: <16057804.MGYHJF1o0G@tinlaptop>
Content-Type: text/plain; charset="us-ascii"

Hi Alessandro,

 I'm not sure I understand your question and I don't know Google SketchUp, but
if you know which edges should form your line loop and you know the starting
and ending vertex number for each of these edges, then it should be quite
simple:
  1) pick the first edge from a loop

  2) the following edge must be such that its starting vertex number is the
same as the ending vertex number of the previous edge. If that's not the case,
add the new edge in the for loop with a minus sign (i.e. flip it)
 Here I suppose that the new edge you're adding has at least one vertex common
with the prevous edge

 3) if there are any edges left to process (the loop is still not closed),
repeat 2)

 I don't know if the edges coming from SketchUp are already ordered and
they're just not oriented, or if they are not ordered at all. In the latter
case, you'll have to modify step 2) to search for suitable candidate edges
every time you want to increment your line loop.

 Best regards,

   Martin Vymazal
   
=end

class Float
	def round_to(x)
		(self*10**x).round.to_f/10**x
	end 
end


def esporta
	param = ["lc param"]
	defaults = [0.1]
	lc = UI.inputbox(param,defaults,"TECLA: Gmsh exporter")
	@model = Sketchup.active_model
    # get selected/all entities of the active model
	if @model.selection.empty?
		@entities = @model.entities
	else
		@entities = @model.selection
	end

	if @entities.length == 0
		UI.messagebox("Nothing to do!")
		return nil
	end
	# define the filename
	@modelName = "Untitled" if (modelName = File.basename(@model.path, ".skp")) == ""
	@model.start_operation("GMSH export preparation", true)
	
	acapo = "\n"   # variabile per il ritorno a capo nelle stringhes
	
	str = "/*********************************************************************
 *
 *  Gmsh *.geo file
 *  generated by TECLA - (c) ing. Alessandro Barracco
 *
 *  Variables, elementary entities (points, lines, surfaces), physical
 *  entities (points, lines, surfaces)
 *
 *********************************************************************/

// The simplest construction in Gmsh's scripting language is the
// `affectation'. The following command defines a new variable `lc':" + acapo

lc = lc[0].to_f

str += "lc = #{lc}" + ";" + acapo
@count = 1
str += "// This variable can then be used in the definition of Gmsh's simplest
// `elementary entity', a `Point'. A Point is defined by a list of
// four numbers: three coordinates (X, Y and Z), and a characteristic
// length (lc) that sets the target element size at the point:" + acapo
	# ------------------------------------------------------------------
	# NODI
	# ------------------------------------------------------------------
	@nodi = []
	@entities.each {|e|
		if e.class == Sketchup::Edge
			@vertici = e.vertices
			0.upto(@vertici.length - 1) do |v|
				x = @vertici[v].position.x.to_m
				y = @vertici[v].position.y.to_m
				z = @vertici[v].position.z.to_m
				@nodi << [x,y,z]	
			end
		end
	}
	@nodi.uniq! # rimuovo i vertici duplicati
	0.upto(@nodi.length - 1) do |n|
		x = @nodi[n][0]
		y = @nodi[n][1]
		z = @nodi[n][2]
		str += "Point(#{@count}) = {#{x}, #{y}, #{z}, lc}"
		str += ";" + acapo
		@count += 1
	end
	# ------------------------------------------------------------------


=begin
THE FOLLOWING IS THE PART I'M NOT ABLE TO SOLVE.... a GMSH user tried to help me and send me this post:

"if you know which edges should form your line loop and you know the starting
and ending vertex number for each of these edges, then it should be quite
simple:
  1) pick the first edge from a loop

  2) the following edge must be such that its starting vertex number is the
same as the ending vertex number of the previous edge. If that's not the case,
add the new edge in the for loop with a minus sign (i.e. flip it)
 Here I suppose that the new edge you're adding has at least one vertex common
with the prevous edge

 3) if there are any edges left to process (the loop is still not closed),
repeat 2)"

I CAN IDENTIFY ALL THE EDGES OF THE LOOP but with SIGN NOT CORRECT!

=end
	
	# ------------------------------------------------------------------
	# BORDI
	# ------------------------------------------------------------------
	@lati = []
	@bordi = []
	@entities.each {|e|
		if e.class == Sketchup::Edge
			@lati << e
		end
	}
	0.upto(@lati.length - 1) do |v|
		str += "Line(#{@count}) = {"
		v1 = @lati[v].start.position # vertice iniziale
		v2 = @lati[v].end.position # vertice finale
		# conversione delle coordinate x,y,z di ogni nodo da m a float
		v1 = [v1.x.to_m, v1.y.to_m, v1.z.to_m]
		v2 = [v2.x.to_m, v2.y.to_m, v2.z.to_m]
		# ricerca dei nodi corrispondenti....
		0.upto(@nodi.length - 1) do |m|
			if @nodi[m] == v1
				@n1 = m + 1
				str += "#{@n1},"
			end
		end
		0.upto(@nodi.length - 1) do |m|
			if @nodi[m] == v2
				@n2 = m + 1
				str += "#{@n2}"
			end
		end
		str += "};" + acapo
		@bordi << [@lati[v], v+1, @n1, @n2]
		@count += 1
	end

	puts "@BORDI="
	puts @bordi.inspect
	# ------------------------------------------------------------------

	# ------------------------------------------------------------------
	# LOOP esterno
	# ------------------------------------------------------------------

	str += "// The third elementary entity is the surface. In order to define a
// simple rectangular surface from the four lines defined above, a
// line loop has first to be defined. A line loop is a list of
// connected lines, a sign being associated with each line (depending
// on the orientation of the line):" + acapo
	@loop=[]
	@entities.each {|e|	 # e = entità generica
		if e.class == Sketchup::Face
			primavolta=true
			str += "Line Loop(#{@count}) = {"
			@loop << @count
			lup = e.outer_loop

			for loop in e.loops
				vertices = loop.vertices
				puts "************* #{vertices.inspect} ***********"
			end

			
			lup.edges.each{|l|  # l=lato del loop in questione
				v1 = l.start.position # vertice iniziale
				v2 = l.end.position # vertice finale
				
				# conversione delle coordinate x,y,z di ogni nodo da m a float
				v1 = [v1.x.to_m, v1.y.to_m, v1.z.to_m]
				v2 = [v2.x.to_m, v2.y.to_m, v2.z.to_m]
				# ricerca dei nodi corrispondenti....
				0.upto(@nodi.length - 1) do |m|
					if @nodi[m] == v1
						@n1 = m + 1
					end
				end
				0.upto(@nodi.length - 1) do |m|
					if @nodi[m] == v2
						@n2 = m + 1
					end
				end
				
				#~ puts "Lato: #{l} - n1= #{@n1} - n2= #{@n2} "
				
				
				i=0
				@lati.each{|b|
					if b==l
						p1 = @bordi[i][2]
						p2 = @bordi[i][3]
						if primavolta
							str += "#{@bordi[i][1]},"
							primavolta=false
						else
							if @n1==p1
								str += "#{@bordi[i][1]},"
							else
								str += "-#{@bordi[i][1]},"
							end
						end
						#~ puts "Lato: #{@bordi[i][0]}=#{@bordi[i][1]} - n1= #{@bordi[i][2]} - n2= #{@bordi[i][3]} "
					end
					i += 1
				}
				
			}
			str = str[0...-1]
			str += "};" + acapo
			@count += 1
		end
		
	}
	
	
	str += "// We can then define the surface as a list of line loops (only one
// here, since there are no holes--see `t4.geo'):" + acapo

	0.upto(@loop.length - 1) do |p|
		str += "Plane Surface(#{@count}) = {#{@loop[p]}};" + acapo
		@count +=1
	end

	#~ @lup.methods.each{|m| puts m}

	# scrivo il file
	filename = UI.savepanel( "Save GMSH file ...", File.dirname(@model.path) , "#{@modelName}.geo" )
	fileDE = File.new(filename, "w" )
	fileDE.puts(str)
	fileDE.close
	@model.commit_operation
	Sketchup.set_status_text("done")
end
 
if (not file_loaded?("su2gmsh.rb"))
	add_separator_to_menu("Tools")
	UI.menu("Tools").add_item("Export to GMSH *.geo") {esporta}
end
#~ 
file_loaded("su2gmsh.rb")
